import ratelimit, requests, sqlitedict, sys, urllib3, time
import concurrent.futures
from tqdm import tqdm

class BlockedException(Exception):
    pass

RATE_LIMIT=20
USER_AGENT="Your user agent here"

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
@ratelimit.sleep_and_retry
@ratelimit.limits(calls=RATE_LIMIT, period=1)
def _fetch(url):
    try:
        return requests.get(url, headers={
            'user-agent': USER_AGENT,
        })
    except (requests.exceptions.ReadTimeout, requests.exceptions.ConnectionError) as e:
        raise ratelimit.RateLimitException(str(e), 1)

blocks = 0
def fetch(d, url, file):
    try:
        result = d.get(url)
        if result is not None:
            return result
        r = _fetch(url)
        if r.status_code == 429:
            raise BlockedException()
        elif r.status_code not in (200, 404, 400, 403): # 400 is 404. 403 is 'private game'
            print(url, r.status_code)
        if file is not None:
            with open(file, "wb") as f:
                f.write(r.content)
        d[url] = r
        return r
    except BlockedException:
        global blocks
        blocks += 1
        print("429 status code for {}".format(url))
        time.sleep(30)
        if False and blocks >= 20:
            print("Crawler is blocked, 20 urls failed with status 429. Exiting.")
            d.commit()
            sys.exit(1)
        else:
            return fetch(d, url, file)


THREADS = RATE_LIMIT*5
CHUNKSIZE = RATE_LIMIT*50
if __name__ == "__main__":
    ex = concurrent.futures.ThreadPoolExecutor(max_workers = THREADS)
    d = sqlitedict.SqliteDict('ogs.db', autocommit=True)
    def _fetch_and_save_game_id(game_id):
        return game_id, fetch(d, url="https://online-go.com/termination-api/game/{}".format(game_id), file=None)
        return game_id, None
    resume = d.get("resume", 0)
    for chunk_start in tqdm(list(range(resume, 36610670, CHUNKSIZE))):
        start_time = time.time()
        chunk_end = chunk_start+CHUNKSIZE
        for game_id, r in ex.map(_fetch_and_save_game_id, range(chunk_start, chunk_end)):
            pass
        d.commit()
        d["resume"] = chunk_end
        elapsed_time = time.time() - start_time
    d.commit()
